const app = Vue.createApp({
  setup() {
    document.body.removeAttribute('hidden')
  },
  data () {
    return {
      comments: [],
      streamInfo: {},
      maxValues: {},
      subscriberCount: 0,
      manualSubscriberCountBackup: {},
      superChatTotal: {},
      giftReceivedCount: 0,
      firstTimeCommentCount: 0,
      maxFirstTimeCommentCount: 0,
      countedFirstTimeIds: new Set(),
      manualMaxFirstTimeCommentCount: null,

      upVoteUnit: 10, upVotePoint: 2,
      viewerUnit: 10, viewerPoint: 2,
      firstTimeUnit: 1, firstTimePoint: 2,
      subscriberUnit: 1, subscriberPoint: 1,
      superChatUnit: 500, superChatPoint: 1,
      giftUnit: 1, giftPoint: 1,

      subscriberCountBase: null,
      currentServiceId: null,
      meta: undefined,

      isScoreCalculationStopped: false,

      frozen: {
        maxValues: {},
        maxFirstTimeCommentCount: 0,
        superChatTotal: {},
        giftReceivedCount: 0,
        weightedScores: {}
      },

      manualSubscriberCount: null,

      // 通貨レート（2025/05/21時点の三菱UFJ銀行 T.T.B.）
      // https://www.bk.mufg.jp/ippan/kinri/list_j/kinri/kawase.html
      currencyRates: {
        '¥': 1,
        '$': 143.28,
        '£': 189.45,
        'CA$': 102.17,
        'CHF': 173.69,
        'SEK': 14.59,
        'EUR': 161.45,
        'DKK': 21.55,
        'NOK': 13.78,
        'PHP': 2.76,
        'THB': 4.31,
        'AUD': 90.87,
        'HK$': 18.00,
        'INR': 1.85,
        'CNY': 19.69,
        'SG$': 110.67,
        'NT$': 4.43,
        '₩': 0.20,
        'NZ$': 83.66,
        'ZAR': 6.56,
        'CZK': 6.43,
        'MXN': 6.49,
        'TRY': 1.22,
        'RUB': 1.54,
        'HUF': 0.39,
        'PLN': 37.27
      }
    }
  },
  computed: {
    weightedScores() {
      const result = {};
      const useFrozen = this.isScoreCalculationStopped;
      const maxValues = useFrozen ? this.frozen.maxValues : this.maxValues;
      const maxFirstTimeCommentCount = useFrozen ? this.frozen.maxFirstTimeCommentCount : this.maxFirstTimeCommentCount;
      // --- SuperChat合計は常にリアルタイム値を参照する ---
      const superChatTotal = this.superChatTotal;
      // --- ギフト合計も常にリアルタイム値を参照する ---
      const giftReceivedCount = this.giftReceivedCount;

      for (const serviceId in this.streamInfo) {
        const upVote = maxValues[serviceId]?.maxUpVote || 0;
        const viewer = maxValues[serviceId]?.maxViewer || 0;
        const maxFirstTime = maxFirstTimeCommentCount || 0;
        let totalSuperChat = 0;
        for (const unit in superChatTotal) {
          totalSuperChat += superChatTotal[unit];
        }
        const giftCount = giftReceivedCount || 0;

        let subscriberCount = 0;
        if (this.manualSubscriberCount !== null && serviceId === this.currentServiceId) {
          subscriberCount = this.manualSubscriberCount;
        } else if (typeof this.meta !== 'undefined' && this.meta && typeof this.meta.subscriberCount === 'number') {
          subscriberCount = this.meta.subscriberCount;
        } else if (this.streamInfo[serviceId]?.subscriberCount !== undefined) {
          subscriberCount = this.streamInfo[serviceId].subscriberCount;
          if (typeof subscriberCount === 'string') {
            const match = subscriberCount.match(/^([\d,.]+)\s*([kKmM])$/);
            if (match) {
              let num = parseFloat(subscriberCount.replace(/,/g, ''));
              const unit = match[2].toLowerCase();
              if (unit === 'k') num *= 1000;
              if (unit === 'm') num *= 1000000;
              subscriberCount = num;
            } else {
              subscriberCount = parseFloat(subscriberCount.replace(/,/g, ''));
            }
          }
        }
        if (this.subscriberCountBase !== null && serviceId === this.currentServiceId) {
          subscriberCount = subscriberCount - this.subscriberCountBase;
        }

        const weightedMaxUpVote = Math.floor(upVote / this.upVoteUnit) * this.upVotePoint;
        const weightedMaxViewer = Math.floor(viewer / this.viewerUnit) * this.viewerPoint;
        const weightedMaxFirstTimeComments = Math.floor(maxFirstTime / this.firstTimeUnit) * this.firstTimePoint;

        let weightedSubscriberCount;
        if (useFrozen && this.frozen.weightedScores && this.frozen.weightedScores[serviceId]) {
          weightedSubscriberCount = this.frozen.weightedScores[serviceId].weightedSubscriberCount;
        } else {
          weightedSubscriberCount = Math.floor(subscriberCount / this.subscriberUnit) * this.subscriberPoint;
        }
        // --- SuperChatスコアは常にリアルタイム値で計算 ---
        const weightedSuperChatTotal = Math.floor(totalSuperChat / this.superChatUnit) * this.superChatPoint;
        // --- ギフトスコアも常にリアルタイム値で計算 ---
        const weightedGiftReceivedCount = Math.floor(giftCount / this.giftUnit) * this.giftPoint;

          // 合計スコアはストップ時も「SuperChat/ギフトはリアルタイム値」で再計算
        const grandTotal =
          weightedMaxUpVote +
          weightedMaxViewer +
          weightedMaxFirstTimeComments +
          Math.floor(subscriberCount / this.subscriberUnit) * this.subscriberPoint +
          weightedSuperChatTotal +
          weightedGiftReceivedCount;

        result[serviceId] = {
          weightedMaxUpVote: this.formatNumber(weightedMaxUpVote),
          weightedMaxViewer: this.formatNumber(weightedMaxViewer),
          weightedMaxFirstTimeComments: this.formatNumber(weightedMaxFirstTimeComments),
          weightedSubscriberCount: this.formatNumber(weightedSubscriberCount),
          weightedSuperChatTotal: this.formatNumber(weightedSuperChatTotal),
          weightedGiftReceivedCount: this.formatNumber(weightedGiftReceivedCount),
          grandTotal: grandTotal
        };
      }
      return result;
    },
    calculatedMaxUpVote() {
      return this.formatNumber(Math.floor(123 / this.upVoteUnit) * this.upVotePoint);
    },
    calculatedMaxViewer() {
      return this.formatNumber(Math.floor(789 / this.viewerUnit) * this.viewerPoint);
    },
    calculatedMaxFirstTimeComments() {
      return this.formatNumber(Math.floor(78 / this.firstTimeUnit) * this.firstTimePoint);
    },
    calculatedSubscriberCount() {
      return this.formatNumber(Math.floor(110 / this.subscriberUnit) * this.subscriberPoint);
    },
    calculatedSuperChatTotal() {
      return this.formatNumber(Math.floor(123456 / this.superChatUnit) * this.superChatPoint);
    },
    calculatedGiftReceivedCount() {
      return this.formatNumber(Math.floor(987 / this.giftUnit) * this.giftPoint);
    },
    calculatedGrandTotal() {
      const maxUpVote = Math.floor(123 / this.upVoteUnit) * this.upVotePoint;
      const maxViewer = Math.floor(789 / this.viewerUnit) * this.viewerPoint;
      const maxFirstTime = Math.floor(78 / this.firstTimeUnit) * this.firstTimePoint;
      const subscriberCount = Math.floor(110 / this.subscriberUnit) * this.subscriberPoint;
      const superChatTotal = Math.floor(123456 / this.superChatUnit) * this.superChatPoint;
      const giftReceivedCount = Math.floor(987 / this.giftUnit) * this.giftPoint;
      const grandTotal = maxUpVote + maxViewer + maxFirstTime + subscriberCount + superChatTotal + giftReceivedCount;
      return this.formatNumber(grandTotal);
    },
    currentSubscriberCount() {
      // manualSubscriberCountがセットされていればそれを優先
      if (this.manualSubscriberCount !== null && this.manualSubscriberCount !== undefined) {
        return this.manualSubscriberCount;
      }
      let current = 0;
      if (typeof this.meta !== 'undefined' && this.meta && typeof this.meta.subscriberCount === 'number') {
        current = this.meta.subscriberCount;
      } else if (this.currentServiceId && this.streamInfo[this.currentServiceId]?.subscriberCount != null) {
        current = this.streamInfo[this.currentServiceId].subscriberCount;
        if (typeof current === 'string') {
          const match = current.match(/^([\d,.]+)\s*([kKmM])$/);
          if (match) {
            let num = parseFloat(current.replace(/,/g, ''));
            const unit = match[2].toLowerCase();
            if (unit === 'k') num *= 1000;
            if (unit === 'm') num *= 1000000;
            current = num;
          } else {
            current = parseFloat(current.replace(/,/g, ''));
          }
        }
        if (isNaN(current)) current = 0;
      }
      return Number.isFinite(current) ? current : 0;
    },
    subscriberDiff() {
      let current = this.currentSubscriberCount;
      if (this.subscriberCountBase === null || typeof this.subscriberCountBase === 'undefined') {
        return Number.isFinite(current) ? current : 0;
      }
      const diff = current - this.subscriberCountBase;
      return Number.isFinite(diff) ? diff : 0;
    }
  },
  methods: {
    toggleScoreCalculation() {
      if (!this.isScoreCalculationStopped) {
        const rawScores = {};
        for (const serviceId in this.weightedScores) {
          rawScores[serviceId] = { ...this.weightedScores[serviceId] };
          if (typeof rawScores[serviceId].weightedSubscriberCount === 'string') {
            rawScores[serviceId].weightedSubscriberCount = Number(
              rawScores[serviceId].weightedSubscriberCount.replace(/,/g, '')
            );
          }
          if (typeof rawScores[serviceId].grandTotal === 'string') {
            rawScores[serviceId].grandTotal = Number(
              rawScores[serviceId].grandTotal.replace(/,/g, '')
            );
          }
        }
        this.frozen.weightedScores = rawScores;
        this.frozen.maxValues = JSON.parse(JSON.stringify(this.maxValues));
        this.frozen.maxFirstTimeCommentCount = this.maxFirstTimeCommentCount;
        // ストップ時もSuperChat/giftはリアルタイム参照なのでfrozenに保存しない
      }
      this.isScoreCalculationStopped = !this.isScoreCalculationStopped;
    },
    getClassName(comment) {
      if (comment.commentIndex % 2 === 0) {
        return 'comment even'
      }
      return 'comment odd'
    },
    getStyle(comment) {
      return OneSDK.getCommentStyle(comment)
    },
    updateMaxValues(serviceId, upVote, viewer) {
      if (!this.maxValues[serviceId]) {
        this.maxValues[serviceId] = { maxUpVote: 0, maxViewer: 0 };
      }
      this.maxValues[serviceId].maxUpVote = Math.max(this.maxValues[serviceId].maxUpVote, upVote);
      this.maxValues[serviceId].maxViewer = Math.max(this.maxValues[serviceId].maxViewer, viewer);
    },
    getSubscriberCount() {
      if (this.manualSubscriberCount !== null && this.manualSubscriberCount !== undefined) {
        return this.manualSubscriberCount;
      }
      if (typeof this.meta !== 'undefined' && this.meta && typeof this.meta.subscriberCount === 'number') {
        return this.meta.subscriberCount;
      }
      if (this.currentServiceId && this.streamInfo[this.currentServiceId]) {
        let val = this.streamInfo[this.currentServiceId].subscriberCount;
        if (typeof val === 'string') {
          const match = val.match(/^([\d,.]+)\s*([kKmM])$/);
          if (match) {
            let num = parseFloat(val.replace(/,/g, ''));
            const unit = match[2].toLowerCase();
            if (unit === 'k') num *= 1000;
            if (unit === 'm') num *= 1000000;
            val = num;
          } else {
            val = parseFloat(val.replace(/,/g, ''));
          }
        }
        return (typeof val === 'number' && !isNaN(val)) ? val : 0;
      }
      return 0;
    },
    setSubscriberCount(newCount) {
      this.manualSubscriberCount = newCount;
      const serviceId = this.currentServiceId;
      if (serviceId && this.streamInfo[serviceId]) {
        if (this.manualSubscriberCountBackup[serviceId] === undefined) {
          this.manualSubscriberCountBackup[serviceId] = this.streamInfo[serviceId].subscriberCount;
        }
        this.streamInfo[serviceId].subscriberCount = newCount;
      }
    },
    setSubscriberCountBase() {
      if (typeof this.meta !== 'undefined' && this.meta && typeof this.meta.subscriberCount === 'number') {
        this.subscriberCountBase = this.meta.subscriberCount;
      } else {
        this.subscriberCountBase = this.getSubscriberCount() || 0;
      }
    },
    updateFirstTimeCommentCount(comments) {
      let newFirstTime = 0;
      comments.forEach(comment => {
        if (
          comment.data.isFirstTime &&
          !comment.data.isRepeater &&
          !this.countedFirstTimeIds.has(comment.data.id) &&
          comment.data.giftType !== 'giftreceived'
        ) {
          this.countedFirstTimeIds.add(comment.data.id);
          newFirstTime++;
        }
      });

      if (this.manualMaxFirstTimeCommentCount === null) {
        this.firstTimeCommentCount += newFirstTime;
        this.maxFirstTimeCommentCount = Math.max(this.maxFirstTimeCommentCount, this.firstTimeCommentCount);
      } else {
        this.maxFirstTimeCommentCount = this.manualMaxFirstTimeCommentCount + newFirstTime;
        this.manualMaxFirstTimeCommentCount = this.maxFirstTimeCommentCount;
        this.firstTimeCommentCount += newFirstTime;
      }
    },
    updateSuperChatTotal(comments) {
      // キーワード処理なし（常に加算のみ）
      comments.forEach(comment => {
        if (
          comment.data.giftType === 'superchat' &&
          comment.data.price &&
          comment.data.unit &&
          !comment.processed
        ) {
          const amount = parseFloat(comment.data.price);
          let currency = comment.data.unit;
          if (!isNaN(amount)) {
            let effectiveAmount = amount; // キーワード判定なし
            let rate = this.currencyRates[currency] ?? 1;
            if (currency === '₩') rate = rate / 100;
            let yenAmount = effectiveAmount * rate;
            if (yenAmount > 0) {
              yenAmount = Math.floor(yenAmount / this.superChatUnit) * this.superChatUnit;
            } else if (yenAmount < 0) {
              yenAmount = -Math.floor(Math.abs(yenAmount) / this.superChatUnit) * this.superChatUnit;
            }
            if (!this.superChatTotal['¥']) {
              this.superChatTotal['¥'] = 0;
            }
            this.superChatTotal['¥'] += yenAmount;
            this.superChatTotal['¥'] = Math.max(0, this.superChatTotal['¥']);
            comment.processed = true;
          }
        }
      });
    },
    updateGiftReceivedCount(comments) {
      // ストップ中でもギフト合計は常に加算
      comments.forEach(comment => {
        if (
          comment.data.giftType === 'giftreceived' &&
          comment.data.gift !== false &&
          !comment.processed
        ) {
          this.giftReceivedCount++;
          comment.processed = true;
        }
      });
    },
    formatNumber(value) {
      if (value === null || value === undefined || value === '') return '0';
      if (typeof value === 'string') {
        value = value.replace(/,/g, '');
        value = parseFloat(value);
      }
      if (isNaN(value)) return '0';
      return value.toLocaleString('ja-JP');
    },
    getCoefficients() {
      return {};
    },
    saveConfig() {
      const config = {
        upVoteUnit: this.upVoteUnit,
        upVotePoint: this.upVotePoint,
        viewerUnit: this.viewerUnit,
        viewerPoint: this.viewerPoint,
        firstTimeUnit: this.firstTimeUnit,
        firstTimePoint: this.firstTimePoint,
        subscriberUnit: this.subscriberUnit,
        subscriberPoint: this.subscriberPoint,
        superChatUnit: this.superChatUnit,
        superChatPoint: this.superChatPoint,
        giftUnit: this.giftUnit,
        giftPoint: this.giftPoint
      };
      const blob = new Blob([JSON.stringify(config, null, 2)], { type: "application/json" });
      const url = URL.createObjectURL(blob);
      const a = document.createElement("a");
      a.href = url;
      a.download = "config.json";
      a.click();
      URL.revokeObjectURL(url);
    },
    loadConfig() {
      const input = document.createElement("input");
      input.type = "file";
      input.accept = ".json,application/json";
      input.onchange = (e) => {
        const file = e.target.files[0];
        if (!file) return;
        const reader = new FileReader();
        reader.onload = (event) => {
          try {
            const config = JSON.parse(event.target.result);
            if (typeof config.upVoteUnit === "number") this.upVoteUnit = config.upVoteUnit;
            if (typeof config.upVotePoint === "number") this.upVotePoint = config.upVotePoint;
            if (typeof config.viewerUnit === "number") this.viewerUnit = config.viewerUnit;
            if (typeof config.viewerPoint === "number") this.viewerPoint = config.viewerPoint;
            if (typeof config.firstTimeUnit === "number") this.firstTimeUnit = config.firstTimeUnit;
            if (typeof config.firstTimePoint === "number") this.firstTimePoint = config.firstTimePoint;
            if (typeof config.subscriberUnit === "number") this.subscriberUnit = config.subscriberUnit;
            if (typeof config.subscriberPoint === "number") this.subscriberPoint = config.subscriberPoint;
            if (typeof config.superChatUnit === "number") this.superChatUnit = config.superChatUnit;
            if (typeof config.superChatPoint === "number") this.superChatPoint = config.superChatPoint;
            if (typeof config.giftUnit === "number") this.giftUnit = config.giftUnit;
            if (typeof config.giftPoint === "number") this.giftPoint = config.giftPoint;
          } catch (err) {
            alert("設定ファイルの読み込みに失敗しました");
          }
        };
        reader.readAsText(file);
      };
      input.click();
    },
    decrementMaxFirstTimeCommentCount() {
      if (this.manualMaxFirstTimeCommentCount === null) {
        this.manualMaxFirstTimeCommentCount = this.maxFirstTimeCommentCount;
      }
      this.manualMaxFirstTimeCommentCount = Math.max(0, this.manualMaxFirstTimeCommentCount - 1);
      this.maxFirstTimeCommentCount = this.manualMaxFirstTimeCommentCount;
    },
    incrementMaxFirstTimeCommentCount() {
      if (this.manualMaxFirstTimeCommentCount === null) {
        this.manualMaxFirstTimeCommentCount = this.maxFirstTimeCommentCount;
      }
      this.manualMaxFirstTimeCommentCount = this.manualMaxFirstTimeCommentCount + 1;
      this.maxFirstTimeCommentCount = this.manualMaxFirstTimeCommentCount;
    }
  },
  mounted () {
    let cache = new Map()
    commentIndex = 0
    OneSDK.setup({
      permissions: ['comments', 'meta'],
    })

    OneSDK.subscribe({
      action: 'comments',
      callback: (comments) => {
        const newCache = new Map()
        comments.forEach(comment => {
          const index = cache.get(comment.data.id)
          if (isNaN(index)) {
            comment.commentIndex = commentIndex
            newCache.set(comment.data.id, commentIndex)
            ++commentIndex
          } else {
            comment.commentIndex = index
            newCache.set(comment.data.id, index)
          }
        });
        cache = newCache;
        this.comments = comments;

        // ストップ中でもSuperChat/giftは常に加算
        const superChatComments = [];
        comments.forEach(comment => {
          if (comment.data.giftType === 'superchat') {
            superChatComments.push(comment);
          }
        });
        this.updateGiftReceivedCount(comments);
        this.updateSuperChatTotal(superChatComments);

        this.updateFirstTimeCommentCount(comments);
      }
    })

    OneSDK.subscribe({
      action: 'meta',
      callback: (meta) => {
        this.meta = meta.data || meta;

        const serviceId = meta.service.id
        const upVote = meta.data.upVote || 0
        const viewer = meta.data.viewer || 0
        const subscriberCount = meta.data.subscriberCount || 0

        this.currentServiceId = serviceId;

        // 前回値の取得（手動変更があればそれを優先）
        let prev = this.manualSubscriberCountBackup[serviceId] !== undefined
          ? this.manualSubscriberCountBackup[serviceId]
          : this.streamInfo[serviceId]?.subscriberCount;
        let prevNum = prev;
        let newNum = subscriberCount;
        if (typeof prevNum === 'string') {
          const match = prevNum.match(/^([\d,.]+)\s*([kKmM])$/);
          if (match) {
            let num = parseFloat(prevNum.replace(/,/g, ''));
            const unit = match[2].toLowerCase();
            if (unit === 'k') num *= 1000;
            if (unit === 'm') num *= 1000000;
            prevNum = num;
          } else {
            prevNum = parseFloat(prevNum.replace(/,/g, ''));
          }
        }
        if (typeof newNum === 'string') {
          const match = newNum.match(/^([\d,.]+)\s*([kKmM])$/);
          if (match) {
            let num = parseFloat(newNum.replace(/,/g, ''));
            const unit = match[2].toLowerCase();
            if (unit === 'k') num *= 1000;
            if (unit === 'm') num *= 1000000;
            newNum = num;
          } else {
            newNum = parseFloat(newNum.replace(/,/g, ''));
          }
        }

        // metaのsubscriberCountが変化した時だけ手動値をリセット
        if (prevNum !== newNum) {
          this.manualSubscriberCount = null;
          if (this.manualSubscriberCountBackup[serviceId] !== undefined) {
            delete this.manualSubscriberCountBackup[serviceId];
          }
        }

        if (!this.streamInfo[serviceId]) {
          this.streamInfo[serviceId] = {};
        }
        this.streamInfo[serviceId].upVote = upVote;
        this.streamInfo[serviceId].viewer = viewer;
        this.streamInfo[serviceId].subscriberCount = subscriberCount;

        this.updateMaxValues(serviceId, upVote, viewer);
      }
    });

    OneSDK.connect()
  },
})
OneSDK.ready().then(() => {
  app.mount("#container");
})